# Copyright 2013 Sam Kleinman, Cyborg Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import textwrap
import logging

from rstcloth.cloth import Cloth

logger = logging.getLogger("rstcloth")

def fill(string, first=0, hanging=0, wrap=True):
    first_indent = ' ' * first
    hanging_indent = ' ' * hanging

    if wrap is True:
        return textwrap.fill(string,
                             width=72,
                             break_on_hyphens=False,
                             break_long_words=False,
                             initial_indent=first_indent,
                             subsequent_indent=hanging_indent)
    else:
        content = string.split('\n')
        if first == hanging:
            return '\n'.join([ first_indent + line for line in content ])
        elif first > hanging:
            indent_diff = first - hanging
            o = indent_diff * ' '
            o += '\n'.join([ hanging_indent + line for line in content ])
            return o
        elif first < hanging:
            indent_diff = hanging - first
            o = '\n'.join([ hanging_indent + line for line in content ])
            return o[indent_diff:]

def _indent(content, indent):
    if indent == 0:
        return content
    else:
        indent = ' ' * indent
        if isinstance(content, list):
            return [ ''.join([indent, line]) for line in content ]
        else:
            return ''.join([indent, content])

class RstCloth(Cloth):
    def __init__(self):
        self._data = []

    def _add(self, content, block=None):
        if block is not None:
            logger.warning('block "{0}" is no longer supported'.format(block))

        if isinstance(content, list):
            self._data.extend(content)
        else:
            self._data.append(content)

    def newline(self, count=1, block=None):
        if isinstance(count, int):
            if count == 1:
                self._add('')
            else:
                # subtract one because every item gets one \n for free.
                self._add('\n' * (count - 1))
        else:
            raise Exception("Count of newlines must be a positive int.")

    def directive(self, name, arg=None, fields=None, content=None, indent=0, wrap=True, block=None):
        o = [ ]

        o.append('.. {0}::'.format(name))

        if arg is not None:
            o[0] += ' ' + arg

        if fields is not None:
            for k, v in fields:
                o.append(_indent(':' + k + ': ' + str(v), 3))

        if content is not None:
            o.append('')

            if isinstance(content, list):
                o.extend(_indent(content, 3))
            else:
                o.append(_indent(content, 3))

        self._add(_indent(o, indent))

    @staticmethod
    def role(name, value, text=None):
        if isinstance(name, list):
            name = ':'.join(name)

        if text is None:
            return ':{0}:`{1}`'.format(name, value)
        else:
            return ':{0}:`{2} <{1}>`'.format(name, value, text)

    @staticmethod
    def bold(string):
        return '**{0}**'.format(string)

    @staticmethod
    def emph(string):
        return '*{0}*'.format(string)

    @staticmethod
    def pre(string):
        return '``{0}``'.format(string)

    @staticmethod
    def inline_link(text, link):
        return '`{0} <{1}>`_'.format(text, link)

    @staticmethod
    def footnote_ref(name):
        return '[#{0}]'.format(name)

    @staticmethod
    def _paragraph(content, wrap=True):
        return [ i.strip() for i in fill(content, wrap=wrap).split('\n') ]

    def replacement(self, name, value, indent=0, block=None):
        output = '.. |{0}| replace:: {1}'.format(name, value)
        self._add(_indent(output, indent))

    def codeblock(self, content, indent=0, wrap=True, language=None, block=None):
        if language is None:
            o = [ '::', _indent(content, 3) ]
            self._add(_indent(o, indent))
        else:
            self.directive(name='code-block', arg=language, content=content, indent=indent)

    def footnote(self, ref, text, indent=0, wrap=True, block=None):
        self._add(fill('.. [#{0}] {1}'.format(ref, text), indent, indent + 3, wrap))

    def definition(self, name, text, indent=0, wrap=True, bold=False, block=None):
        o = []

        if bold is True:
            name = self.bold(name)

        o.append(_indent(name, indent))
        o.append(fill(text, indent + 3, indent + 3, wrap=wrap))

        self._add(o)

    def li(self, content, bullet='-', indent=0, wrap=True, block=None):
        bullet = bullet + ' '
        hanging_indent_len = indent + len(bullet)
        hanging_indent = ' ' * hanging_indent_len

        if isinstance(content, list):
            content = bullet + '\n'.join(content)
            self._add(fill(content, indent, indent + hanging_indent_len, wrap))
        else:
            content = bullet + fill(content, 0, len(bullet), wrap)
            self._add(fill(content, indent, indent, wrap))

    def field(self, name, value, indent=0, wrap=True, block=None):
        output = [ ':{0}:'.format(name) ]

        if len(name) + len(value) < 60:
            output[0] += ' ' + value
            final = True
        else:
            output.append('')
            final = False

        if wrap is True and final is False:
            content = fill(value, wrap=wrap).split('\n')
            for line in content:
                output.append(_indent(line, 3))

        if wrap is False and final is False:
            output.append(_indent(value, 3))

        for line in output:
            self._add(_indent(line, indent))

    def ref_target(self, name, indent=0, block=None):
        o = '.. _{0}:'.format(name)
        self._add(_indent(o, indent))

    def content(self, content, indent=0, wrap=True, block=None):
        if isinstance(content, list):
            for line in content:
                self._add(_indent(line, indent))
        else:
            lines = self._paragraph(content, wrap)

            for line in lines:
                self._add(_indent(line, indent))

    def title(self, text, char='=', block=None):
        line = char * len(text)
        self._add([line, text, line])

    def heading(self, text, char, indent, block=None):
        self._add(_indent([text, char * len(text)], indent))

    def h1(self, text, indent=0, block=None):
        self.heading(text, char='=', indent=indent)

    def h2(self, text, indent=0, block=None):
        self.heading(text, char='-', indent=indent)

    def h3(self, text, indent=0, block=None):
        self.heading(text, char='~', indent=indent)

    def h4(self, text, indent=0, block=None):
        self.heading(text, char='+', indent=indent)

    def h5(self, text, indent=0, block=None):
        self.heading(text, char='^', indent=indent)

    def h6(self, text, indent=0, block=None):
        self.heading(text, char=';', indent=indent)



# Copyright 2012-2013 10gen, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Sam Kleinman, Kay Kim
from __future__ import unicode_literals

import os
import sys
import argparse
import string
import textwrap

import yaml

from rstcloth import RstCloth

def normalize_cell_height(rowdata):
    """
    Normalize cells in the rowdata so that each cell has the same height;
    more specifically, ensure that each cell data list spans the same number of
    lines when printed as others in the row. Mutates 'rowdata' in place by only
    appeneding rather than resetting the reference.
    """
    maxlines = max([ len(cell) for cell in rowdata])

    for cell in rowdata:
        for x in range(maxlines - len(cell)):
            cell.append( ' '  )

def fill(string, first=0, hanging=0):
    first_indent = ' ' * first
    hanging_indent = ' ' * hanging

    return textwrap.fill(string,
                         width=72,
                         initial_indent=first_indent,
                         subsequent_indent=hanging_indent)

###################################
#
# Generating parts of the table.

class TableData(object):
    def __init__(self, num_columns=None):
        self.num_columns = num_columns
        self.num_rows = 0
        self.header = None
        self.rows = []
        self.final = False

    def _add(self, row, location='row'):
        if self.final == True:
            pass
        else:
            if location == 'header':
                self.header.append(row)
            elif location == 'row':
                self.rows.append(row)
            else:
                raise Exception('tables can only have headers or rows.')

    def _columns(self, num_columns):
        if self.num_columns is None:
            self.num_columns = num_columns
            return True
        elif self.num_columns == num_columns:
            return True
        elif num_columns != self.num_columns:
            raise Exception('rows must have the same number of columns as the table.')

    def add_row(self, row):
        if self._columns(len(row)):
            self.num_rows += 1
            self._add({ self.num_rows: row }, location='row')

    def add_header(self, row):
        if self._columns(len(row)):
            if self.header is None:
                self.header = []

            o = []

            for i in row:
                o.append(i)

            self._add({0: o}, location='header')

    def finalize(self):
        self.final = True

class YamlTable(TableData):
    def __init__(self, inputfile, num_columns=None):
        super(YamlTable, self).__init__(num_columns)
        self.inputfile = inputfile
        self.format = None

        layout, meta, content = self.read_data(inputfile)
        self.layout = layout
        self.parse_table(layout, meta, content)

    def parse_table(self, layout, meta, content):
        if hasattr(meta, 'format'):
            self.format = meta.format

        if layout.header:
            header = []
            for cell in layout.header:
                header.append(eval(cell))

            self.add_header(header)

        for row in layout.rows:
            cells = []
            for cell in row.values():
                for c in cell:
                    cells.append(eval(c))

            self.add_row(cells)

    def read_data(self, datafile):
        with open(datafile, "rt") as f:
            parsed = yaml.load_all(f)
            layout = dict2obj(parsed.next())
            meta = dict2obj(parsed.next())
            content = dict2obj(parsed.next())

        if layout.section != 'layout':
            exit('layout document in "' + datafile + '" is malformed.')
        elif meta.section != 'meta':
            exit('meta document in "' + datafile + '" is malformed.')
        elif content.section != 'content':
            exit('content document in "' + datafile + '" is malformed.')

        return layout, meta, content

###################################
#
# Reading and processing input, and rendering table.

class dict2obj(object):
    """
    Converts a dict (from yaml, in this case) into a Python object.
    """

    def __init__(self, d):
        self.__dict__['d'] = d

    def __getattr__(self, key):
        value = self.__dict__['d'][key]
        if type(value) == type({}):
            return dict2obj(value)

        return value

###################################
#
# Interaction

class OutputTable(object):
    pass

class RstTable(OutputTable):
    def __init__(self, imported_table):
        self.columnwidths = []
        self.tempcolumnwidths = []

        self.table = imported_table

        self.process_table_content()
        self.output = self.render_table()

    ###################################
    #
    # Flexibility for tables of different sizes.

    def _check_column_width(self, rowdata):
        """
        Compares the cell widths of the row with the curren max cell
        width in the global variables. Then updates
        ``tempcolumnwidths``. ``tempcolumnwidths`` variable maintains
        the running max width of each column.
        """
        if rowdata == [{}]:
            return False

        for row in rowdata:
            if self.tempcolumnwidths:
                max_widths = self.tempcolumnwidths.pop()
            else:
                max_widths = None

            current_widths = []

            for line in row.values()[0]:
                _width = []
                for subline in line.split('\n'):
                    _width.append(len(subline))
                current_widths.append(max(_width))

            if max_widths is None:
                self.tempcolumnwidths.append(current_widths)
            else:
                o = []
                for this, running in zip(current_widths, max_widths):
                    o.append(max(running, this))
                max_widths = o

                self.tempcolumnwidths.append(max_widths)

        return True

    ###################################
    #
    # Building the table representation

    def _get_row_line(self, delim='-'):
        """
        Produces and returns row deliminiters for restructured text tables.
        """
        return '+' + delim + str(delim + '+' + delim).join([ delim * width for width in self.columnwidths ]) + delim + '+'

    def _get_row(self, rowdata):
        """
        Returns rows given ``rowdata`` properly formated for restructured text tables.
        """
        rowlines = []
        for line in zip(*rowdata):
            if len(rowlines) > 0:
                rowlines.append('\n')

            rowlines.append( '| ' + ' | '.join([ line[idx].ljust(self.columnwidths[idx]) for idx in range(len(line)) ]) + ' |' )

        return ''.join(rowlines)

    def _get_header_row(self, header):
        o = '| '

        cols = range(len(self.columnwidths))
        for header, rowidths, idx in zip(header, self.columnwidths, cols):
            o += header.ljust(self.columnwidths[idx]) + ' | '

        return o

    def process_table_content(self):
        self.table_data = []
        if not hasattr(self, 'num_columns'):
            self.num_columns = None
        # Compare cell widths of the header  with the
        # max cell widths stored in the global var tempcolumnwidths
        # and swap out value(s) if necessary.
        if self.table.header is not None:
            if self._check_column_width(self.table.header):
                self.table.header = self.table.header[0][0]

            if self.num_columns is None:
                self.num_columns = len(self.table.header)

        for row, index in zip(self.table.rows, range(len(self.table.rows))):
            if not row:
                continue

            parsed_row = []
            index = index + 1

            for line in row[index]:
                parsed_row.append(line.split('\n'))

            # process the data to ensure the table is big enough.
            if self.num_columns is None:
                self.num_columns = len(self.table.rows)

            self._check_column_width(self.table.rows)

            normalize_cell_height(parsed_row)

            # add the processed data to the table
            self.table_data.append(parsed_row)

        if len(self.tempcolumnwidths) > 0:
            self.columnwidths = self.tempcolumnwidths.pop()

    def render_table(self):
        o = []
        o.append(self._get_row_line())

        if self.table.header is not None:
            o.append(self._get_header_row(self.table.header))
            o.append(self._get_row_line('='))

        for row in self.table_data:
            o.append(self._get_row(row))
            o.append(self._get_row_line())

        return o

###################################
#
# Outputs a list-table

class ListTable(OutputTable):
    def __init__(self, imported_table, widths=None, indent=0):
        self.table = imported_table
        self.indent = indent

        if widths is not None:
            self.widths = [ str(i) for i in widths ]
        else:
            self.widths = None

        self.r = RstCloth()
        self._render_table()
        self.output = self.r.data

    def _render_table(self):
        b = '_all'

        rows = []
        _fields = []
        if self.table.header is not None:
            _fields.append(('header-rows', '1'))
            rows.append(self.table.header[0])
            idx = 0
        else:
            idx = 1

        if self.widths is not None:
            _fields.append(('widths', ' '.join(self.widths)))

        rows.extend(self.table.rows)

        self.r.directive('list-table', fields=_fields, indent=self.indent, block=b)

        self.r.newline(block=b)

        for row in rows:
            r = row[idx]

            self.r.li(r[0], bullet='* -', indent=self.indent + 3, wrap=False, block=b)
            self.r.newline(block=b)

            for cell in r[1:]:
                self.r.li(cell, bullet='  -',  indent=self.indent + 3, wrap=False, block=b)
                self.r.newline(block=b)

            idx += 1

###################################
#
# Outputs an HTML-table
# Currently, does NO processing of the data
# such as to handle links or to handle
# code-block directives

class HtmlTable(OutputTable):
    def __init__(self, imported_table):
        self.tags = { 'tr': '<tr>',
            'th': '<th>',
            'td': '<td>',
            'table': '<table>'
            }

        self.table = imported_table
        self.output = self.render_table()

    def render_table(self):
        o = [self.tags['table']]

        if self.table.header is not None:
            o.append(self._process_html_row(self.tags['tr'], self.tags['th'], self.table.header) )

        for row in self.table.rows:
            o.append(self._process_html_row(self.tags['tr'], self.tags['td'], row.values()))

        o.append(self._get_ending_tag(self.tags['table']))
        return o

    def _process_html_row(self, tag, tagchild, rowdata):
        row=[]
        row.append(tag)
        row.append("\n")

        if tagchild is None:
            row.append(rowdata)
        else:
            for data in rowdata:
                for cell in data:
                    row.append(self._process_html_row(tagchild, None, cell))
                    row.append("\n")
        row.append("\n")
        row.append(self._get_ending_tag(tag))

        return ''.join(row)

    def _get_ending_tag(self, tag):
        return string.join(tag.split('<', 1), '</')

class TableBuilder(object):
    def __init__(self, table):
        self.output = table.output

    def write(self, outputfile=None):
        if outputfile is None:
            outputfile = get_outputfile(self.inputfile)

        dirpath = os.path.dirname(outputfile)
        if os.path.isdir(dirpath) is False:
            try:
                os.makedirs(dirpath)
            except OSError:
                logger.debug('{0} exists. ignoring.'.format(dirpath))

        with open(outputfile, 'w') as f:
            for line in self.output:
                f.write(line + '\n')

    def print_table(self):
        for line in self.output:
            print(line)

###################################
#
# Interface.

def get_outputfile(inputfile, outputfile):
    if outputfile is None:
        return inputfile.rsplit('.')[0] + '.rst'
    else:
        return outputfile

formats = { 'rst': RstTable,
            'list': ListTable,
            'html': HtmlTable }

def user_input():
    parser = argparse.ArgumentParser('YAML to (RST/HTML) Table Builder')
    parser.add_argument('input', nargs='?', help='path of source yaml file.')

    output_help = 'path of output file. by default, the input file name with an ".rst" extension.'
    parser.add_argument('output', nargs='?', default=None, help=output_help)
    parser.add_argument('--type', '-t', choices=formats.keys(), default='rst',
                        help='output table format.')

    return parser.parse_args()

def main():
    ui = user_input()

    table_data = YamlTable(ui.input)

    if table_data.format is None:
        table_data.format = ui.type

    table = TableBuilder(formats[table_data.format](table_data))

    table.write(get_outputfile(ui.input, ui.output))

if __name__ == '__main__':
    main()



# Copyright 2013 Sam Kleinman, Cyborg Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging

logger = logging.getLogger("rstcloth.cloth")

class Cloth(object):
    def print_content(self, block_order=None):
        if block_order is not None:
            logger.warning('block_order "{0}" is no longer supported'.format(block_order))

        print('\n'.join(self._data))

    def print_block(self, block='_all'):
        logger.warning('print_block is no longer supported')

    def write(self, filename, block_order=None):
        if block_order is not None:
            logger.warning('block_order "{0}" is no longer supported'.format(block_order))

        dirpath = filename.rsplit('/', 1)[0]
        if os.path.isdir(dirpath) is False:
            try:
                os.makedirs(dirpath)
            except OSError:
                logger.info('{0} exists. ignoring.'.format(dirpath))

        with open(filename, 'w') as f:
            f.write('\n'.join(self._data))
            f.write('\n')

    def write_block(self, filename, block='_all'):
        logger.warning('write_block is no longer supported')

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self):
        raise AttributeError('cannot set the RstCloth.data attribute directly')



from unittest import TestCase
from rstcloth.rstcloth import RstCloth

class TestRstCloth(TestCase):
    @classmethod
    def setUp(self):
        self.r = RstCloth()

    def test_adding_without_blocks(self):
        self.r._add('foo')
        self.assertEqual(self.r.data[0], 'foo')

    def test_newline(self):
        self.r.newline()
        self.assertEqual(len(self.r.data), 1)

    def test_multi_newline(self):
        self.r.newline(count=4)
        self.assertEqual(len(self.r.data[0]), 4 - 1)

    def test_directive_simple(self):
        self.r.directive('test', block='d0')
        self.assertEqual(self.r.data[0], '.. test::')

    def test_directive_arg_named(self):
        self.r.directive('test', arg='what', block='d3')
        self.assertEqual(self.r.data[0], '.. test:: what')

    def test_directive_arg_positional(self):
        self.r.directive('test', 'what', block='d1')
        self.assertEqual(self.r.data[0], '.. test:: what')

    def test_directive_fields(self):
        self.r.directive('test', fields=[('a', 'b')], block='d2')
        self.assertEqual(self.r.data[0], '.. test::')
        self.assertEqual(self.r.data[1], '   :a: b')

    def test_directive_fields_with_arg(self):
        self.r.directive('test', arg='what', fields=[('a', 'b')], block='d4')
        self.assertEqual(self.r.data[0], '.. test:: what')
        self.assertEqual(self.r.data[1], '   :a: b')

    def test_directive_fields_multiple(self):
        self.r.directive('test', fields=[('a', 'b'), ('c', 'd')], block='d5')
        self.assertEqual(self.r.data[0], '.. test::')
        self.assertEqual(self.r.data[1], '   :a: b')
        self.assertEqual(self.r.data[2], '   :c: d')

    def test_directive_fields_multiple_arg(self):
        self.r.directive('test', arg='new', fields=[('a', 'b'), ('c', 'd')], block='d6')
        self.assertEqual(self.r.data[0], '.. test:: new')
        self.assertEqual(self.r.data[1], '   :a: b')
        self.assertEqual(self.r.data[2], '   :c: d')

    def test_directive_content(self):
        self.r.directive('test', content='string', block='d7')
        self.assertEqual(self.r.data[0], '.. test::')
        self.assertEqual(self.r.data[1], '')
        self.assertEqual(self.r.data[2], '   string')

    def test_directive_with_multiline_content(self):
        self.r.directive('test', content=['string', 'second'], block='d8')
        self.assertEqual(self.r.data[0], '.. test::')
        self.assertEqual(self.r.data[1], '')
        self.assertEqual(self.r.data[2], '   string')
        self.assertEqual(self.r.data[3], '   second')


    def test_directive_simple_indent(self):
        self.r.directive('test', indent=3, block='di0')
        self.assertEqual(self.r.data, ['   .. test::'])

    def test_directive_arg_named_indent(self):
        self.r.directive('test', arg='what', indent=3, block='di3')
        self.assertEqual(self.r.data, ['   .. test:: what'])

    def test_directive_arg_positional_indent(self):
        self.r.directive('test', 'what', indent=3, block='di1')
        self.assertEqual(self.r.data, ['   .. test:: what'])

    def test_directive_fields_indent(self):
        self.r.directive('test', fields=[('a', 'b')], indent=3, block='di2')
        self.assertEqual(self.r.data, ['   .. test::', '      :a: b'])

    def test_directive_fields_with_arg_indent(self):
        self.r.directive('test', arg='what', fields=[('a', 'b')], indent=3, block='di4')
        self.assertEqual(self.r.data, ['   .. test:: what', '      :a: b'])

    def test_directive_fields_multiple_indent(self):
        self.r.directive('test', indent=3, fields=[('a', 'b'), ('c', 'd')], block='di5')
        self.assertEqual(self.r.data, ['   .. test::', '      :a: b', '      :c: d'])

    def test_directive_fields_multiple_arg_indent(self):
        self.r.directive('test', arg='new', indent=3, fields=[('a', 'b'), ('c', 'd')], block='di6')
        self.assertEqual(self.r.data, ['   .. test:: new', '      :a: b', '      :c: d'])

    def test_directive_content_indent(self):
        self.r.directive('test', content='string', indent=3, block='di7')
        self.assertEqual(self.r.data, ['   .. test::', '   ', '      string'])

    def test_directive_with_multiline_content_indent(self):
        self.r.directive('test', indent=3, content=['string', 'second'], block='di8')
        self.assertEqual(self.r.data, ['   .. test::', '   ', '      string', '      second'])

    def test_single_role_no_text(self):
        ret = self.r.role('test', 'value')
        self.assertEqual(ret, ':test:`value`')

    def test_multi_role_no_text(self):
        ret = self.r.role(['test', 'role'], 'value')
        self.assertEqual(ret, ':test:role:`value`')

    def test_single_role_text(self):
        ret = self.r.role('test', 'value', 'link')
        self.assertEqual(ret, ':test:`link <value>`')

    def test_multi_role_text(self):
        ret = self.r.role(['test', 'role'], 'value', 'link')
        self.assertEqual(ret, ':test:role:`link <value>`')

    def test_single_role_no_text_args(self):
        ret = self.r.role(name='test', value='value')
        self.assertEqual(ret, ':test:`value`')

    def test_multi_role_no_text_args(self):
        ret = self.r.role(name=['test', 'role'], value='value')
        self.assertEqual(ret, ':test:role:`value`')

    def test_single_role_text_args(self):
        ret = self.r.role(name='test', value='value', text='link')
        self.assertEqual(ret, ':test:`link <value>`')

    def test_multi_role_text_args(self):
        ret = self.r.role(name=['test', 'role'], value='value', text='link')
        self.assertEqual(ret, ':test:role:`link <value>`')

    def test_bold(self):
        ret = self.r.bold('text')
        self.assertEqual(ret, '**text**')

    def test_emph(self):
        ret = self.r.emph('text')
        self.assertEqual(ret, '*text*')

    def test_pre(self):
        ret = self.r.pre('text')
        self.assertEqual(ret, '``text``')

    def test_inline_link(self):
        ret = self.r.inline_link('text', 'link')
        self.assertEqual(ret, '`text <link>`_')

    def test_footnote_ref(self):
        ret = self.r.footnote_ref('name')
        self.assertEqual(ret, '[#name]')

    def test_codeblock_simple(self):
        self.r.codeblock('ls -lha', block='cb0')
        self.assertEqual(self.r.data, ['::', '   ls -lha'])

    def test_codeblock_with_language(self):
        self.r.codeblock('ls -lha', language='shell',block='cb1')
        self.assertEqual(self.r.data, ['.. code-block:: shell', '', '   ls -lha'])

    def test_footnote(self):
        self.r.footnote('footsnotes', 'text of the note', block='fn0')
        self.assertEqual(self.r.data[0], '.. [#footsnotes] text of the note')

    def test_footnote_with_indent(self):
        self.r.footnote('footsnotes', 'text of the note', block='fn1', indent=3)
        self.assertEqual(self.r.data[0], '   .. [#footsnotes] text of the note')

    def test_footnote_with_wrap(self):
        self.r.footnote('footsnotes', 'the ' * 40, block='fn2', wrap=True)
        self.assertEqual(self.r.data[0],
                         '.. [#footsnotes]' + ' the' * 14 + '\n  ' + ' the' * 17 + '\n  ' + ' the' * 9)

    def test_definition(self):
        self.r.definition('defitem', 'this is def text', block='dfn0')
        self.assertEqual(self.r.data, ['defitem', '   this is def text'])

    def test_definition_with_indent(self):
        self.r.definition('defitem', 'this is def text', indent=3, block='dfn1')
        self.assertEqual(self.r.data, ['   defitem', '      this is def text'])

    def test_title_default(self):
        self.r.title('test text', block='hd0')
        self.assertEqual(self.r.data, ['=========', 'test text', '========='])

    def test_title_alt(self):
        self.r.title('test text', char='-', block='hd1')
        self.assertEqual(self.r.data, ['---------', 'test text', '---------'])

    def test_heading_one(self):
        self.r.heading('test heading', char='-', indent=0, block='hd2')
        self.assertEqual(self.r.data, ['test heading', '------------'])

    def test_heading_two(self):
        self.r.heading('test heading', char='^', indent=0, block='hd3')
        self.assertEqual(self.r.data, ['test heading', '^^^^^^^^^^^^'])

    def test_h1(self):
        self.r.h1('test', block='hd4')
        self.assertEqual(self.r.data, ['test', '===='])

    def test_h2(self):
        self.r.h2('test', block='hd5')
        self.assertEqual(self.r.data, ['test', '----'])

    def test_h3(self):
        self.r.h3('test', block='hd6')
        self.assertEqual(self.r.data, ['test', '~~~~'])

    def test_h4(self):
        self.r.h4('test', block='hd7')
        self.assertEqual(self.r.data, ['test', '++++'])

    def test_h5(self):
        self.r.h5('test', block='hd8')
        self.assertEqual(self.r.data, ['test', '^^^^'])

    def test_h6(self):
        self.r.h6('test', block='hd9')
        self.assertEqual(self.r.data, ['test', ';;;;'])

    def test_replacement(self):
        self.r.replacement('foo', 'replace-with-bar', block='sub0')
        self.assertEqual(self.r.data, ['.. |foo| replace:: replace-with-bar'])

    def test_replacement_with_indent(self):
        self.r.replacement('foo', 'replace-with-bar', indent=3, block='sub1')
        self.assertEqual(self.r.data, ['   .. |foo| replace:: replace-with-bar'])

    def test_li_simple(self):
        self.r.li('foo', block='li0')
        self.assertEqual(self.r.data, ['- foo'])

    def test_li_simple_indent(self):
        self.r.li('foo', indent=3, block='li1')
        self.assertEqual(self.r.data, ['   - foo'])

    def test_li_simple_alt(self):
        self.r.li('foo', bullet='*', block='li2')
        self.assertEqual(self.r.data, ['* foo'])

    def test_li_simple_alt_indent(self):
        self.r.li('foo', bullet='*', indent=3, block='li3')
        self.assertEqual(self.r.data, ['   * foo'])

    def test_li_complex(self):
        self.r.li(['foo', 'bar'], block='li0')
        self.assertEqual(self.r.data, ['- foo bar'])

    def test_li_complex_indent(self):
        self.r.li(['foo', 'bar'], indent=3, block='li1')
        self.assertEqual(self.r.data, ['   - foo bar'])

    def test_li_complex_alt(self):
        self.r.li(['foo', 'bar'], bullet='*', block='li2')
        self.assertEqual(self.r.data, ['* foo bar'])

    def test_li_complex_alt_indent(self):
        self.r.li(['foo', 'bar'], bullet='*', indent=3, block='li3')
        self.assertEqual(self.r.data, ['   * foo bar'])

    def test_field_simple(self):
        self.r.field('fname', 'fvalue', block='fld0')
        self.assertEqual(self.r.data, [':fname: fvalue'])

    def test_field_long_simple(self):
        self.r.field('fname is fname', 'fvalue', block='fld1')
        self.assertEqual(self.r.data, [':fname is fname: fvalue'])

    def test_field_simple_long(self):
        self.r.field('fname', 'v' * 54, block='fld2')
        self.assertEqual(self.r.data, [':fname: ' + 'v' * 54])

    def test_field_simple_long_long(self):
        self.r.field('fname', 'v' * 55, block='fld3')
        self.assertEqual(self.r.data, [':fname:', '', '   ' + 'v' * 55])

    def test_field_indent_simple(self):
        self.r.field('fname', 'fvalue', indent=3, block='fld4')
        self.assertEqual(self.r.data, ['   :fname: fvalue'])

    def test_field_indent_long_simple(self):
        self.r.field('fname is fname', 'fvalue', indent=3, block='fld5')
        self.assertEqual(self.r.data, ['   :fname is fname: fvalue'])

    def test_field_indent_simple_long(self):
        self.r.field('fname', 'v' * 54, indent=3, block='fld6')
        self.assertEqual(self.r.data, ['   :fname: ' + 'v' * 54])

    def test_field_indent_simple_long_long(self):
        self.r.field('fname', 'v' * 55, indent=3, block='fld7')
        self.assertEqual(self.r.data, ['   :fname:', '   ', '      ' + 'v' * 55])

    def test_field_wrap_simple(self):
        self.r.field('fname', 'the ' * 100, block='fld8')
        self.assertEqual(self.r.data, [':fname:', '', '  ' + ' the' * 18, '  ' + ' the' * 18, '  ' + ' the' * 18, '  ' + ' the' * 18, '  ' + ' the' * 18, '  ' + ' the' * 10])

    def test_field_wrap_indent_simple(self):
        self.r.field('fname', 'the ' * 100, indent=3, block='fld8')
        self.assertEqual(self.r.data, ['   :fname:', '   ', '     ' + ' the' * 18, '     ' + ' the' * 18, '     ' + ' the' * 18, '     ' + ' the' * 18, '     ' + ' the' * 18, '     ' + ' the' * 10])

    def test_content_string(self):
        self.r.content('this is sparta', block='ct0')
        self.assertEqual(self.r.data, ['this is sparta'])

    def test_content_list(self):
        self.r.content(['this is sparta', 'this is spinal tap'], block='ct1')
        self.assertEqual(self.r.data, ['this is sparta', 'this is spinal tap'])

    def test_content_indent_string(self):
        self.r.content('this is sparta', indent=3, block='ct2')
        self.assertEqual(self.r.data, ['   this is sparta'])

    def test_content_indent_list(self):
        self.r.content(['this is sparta', 'this is spinal tap'], indent=3, block='ct3')
        self.assertEqual(self.r.data, ['   this is sparta', '   this is spinal tap'])

    def test_content_long(self):
        self.r.content('the ' * 100, block='ct4')
        self.assertEqual(self.r.data, [ 'the' + ' the' * 17, 'the ' * 17 + 'the', 'the ' * 17 + 'the', 'the ' * 17 + 'the', 'the ' * 17 + 'the', 'the ' * 9 + 'the' ])

    def test_ontent_indent_long(self):
        self.r.content('the ' * 100, indent=3 ,block='ct5')
        self.assertEqual(self.r.data, [ '   the' + ' the' * 17, "   " + 'the ' * 17 + 'the', '   ' + 'the ' * 17 + 'the', '   ' + 'the ' * 17 + 'the', '   ' + 'the ' * 17 + 'the', '   ' + 'the ' * 9 + 'the' ])

    def test_ontent_indent_long_nowrap(self):
        self.r.content('the ' * 100, wrap=False, indent=3 ,block='ct5')
        self.assertEqual(self.r.data, [ '   ' + 'the ' * 99 + 'the'])

    def test_ref_target_named(self):
        self.r.ref_target(name="foo-are-magic-ref0", block='ref0')
        self.assertEqual(self.r.data, ['.. _foo-are-magic-ref0:'])

    def test_ref_target_unnamed(self):
        self.r.ref_target("foo-are-magic-ref1", block='ref1')
        self.assertEqual(self.r.data, ['.. _foo-are-magic-ref1:'])

    def test_ref_target_named_with_indent(self):
        self.r.ref_target(name="foo-are-magic-ref2", indent=3, block='ref2')
        self.assertEqual(self.r.data, ['   .. _foo-are-magic-ref2:'])

    def test_ref_target_unnamed_wo_indent(self):
        self.r.ref_target("foo-are-magic-ref3", 3, block='ref3')
        self.assertEqual(self.r.data, ['   .. _foo-are-magic-ref3:'])



from setuptools import setup
import rstcloth

setup(
    name='rstcloth',
    description='A simple Python API for generating RestructuredText.',
    version=rstcloth.__version__,
    author='Sam Kleinman',
    author_email='sam@tychoish.com',
    license='Apache',
    url='http://cyborginstitute.org/projects/rstcloth',
    packages=['rstcloth'],
    setup_requires=['nose'],
    test_suite='test',
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Documentation',
        'Topic :: Text Processing',
        ],
    entry_points={
        'console_scripts': [
            'rstable = rstcloth.table:main',
            ],
        }
    )
